/***************************************************************************************/
/* Klasse CLinkedList       (CLinkedList.cpp)                                          */
/*                                                                                     */
/* Kurzbeschreibung                                                                    */
/*  Klasse fuer eine doppelt verkettete Liste (engl. linked list)                      */
/*                                                                                     */
/* Aenderungen/Ergaenzungen                                                            */
/*                                                                                     */
/* Entwickler: Buchgeher Stefan                                                        */
/* Entwicklungsbeginn dieser Klasse: 10. Oktober 2006                                  */
/* Funktionsfaehig seit: 11. Oktober 2006                                              */
/* Letzte Bearbeitung: 24. Januar 2007                                                 */
/***************************************************************************************/

/****************** Include-Dateien ****************************************************/
#include "CLinkedList.h"
#include "CNode.h"
#include <iostream.h>


/****************** Konstruktoren ******************************************************/

/***************************************************************************************/
/* CLinkedList (Default-Konstruktor):                                                  */
/*                                                                                     */
/* Aufgaben:                                                                           */
/*  - Die Member-Variablen mit 0 initialisieren                                        */
/*                                                                                     */
/* Veraenderte Member-Variablen:                                                       */
/*  - m_pRoot                                                                          */
/*  - m_pEnd                                                                           */
/*  - m_pActuell                                                                       */
/***************************************************************************************/
CLinkedList::CLinkedList()
{
    m_pRoot = 0;
    m_pEnd = 0;
    m_pActuell = 0;
}


/***************************************************************************************/
/* CLinkedList(int nValue):                                                            */
/*                                                                                     */
/* Aufgaben:                                                                           */
/*  - Zunaechst alle Member-Variablen initialisieren und                               */
/*  - Den ersten Knoten erzeugen und darin den uebergebenen Wert sichern               */
/*                                                                                     */
/* Uebergabeparameter:                                                                 */
/*  - nValue (Die zu speichernde Zahl)                                                 */
/*                                                                                     */
/* Veraenderte Member-Variablen:                                                       */
/*  - m_pRoot                                                                          */
/*  - m_pEnd                                                                           */
/*  - m_pActuell                                                                       */
/***************************************************************************************/
CLinkedList::CLinkedList(int nValue)
{
    m_pRoot = 0;
    m_pEnd = 0;
    m_pActuell = 0;

    AddNode(nValue);
}


/***************************************************************************************/
/* CLinkedList (const CLinkedList &rhs):     (Kopierkonstruktor)                       */
/*                                                                                     */
/* Aufgabe:                                                                            */
/*  - Kopierkonstruktor                                                                */
/*                                                                                     */
/* Veraenderte Member-Variablen:                                                       */
/*  keine                                                                              */
/***************************************************************************************/
//CLinkedList::CLinkedList (const CLinkedList &rhs)
//{
//    noch nicht vorhanden
//}


/****************** Destruktor *********************************************************/

/***************************************************************************************/
/* ~CLinkedList:                                                                       */
/*                                                                                     */
/* Aufgabe:                                                                            */
/*  Die gesamte Liste (falls vorhanden) vom Speicher entfernen.                        */
/*                                                                                     */
/* Veraenderte Member-Variablen:                                                       */
/*  - m_pRoot                                                                          */
/*  - m_pEnd                                                                           */
/*  - m_pActuell                                                                       */
/***************************************************************************************/
CLinkedList::~CLinkedList()
{
    DeleteList();
}


/****************** Member-Funktionen **************************************************/

/***************************************************************************************/
/* int AddNode(nValue):                                                                */
/*                                                                                     */
/* Aufgabe:                                                                            */
/*  Neuen Knoten (mit dem Wert nValue) an die bestehende Liste anhaengen.              */
/*                                                                                     */
/* Uebergabeparameter:                                                                 */
/*  nValue: Dieser Wert soll in den neu hinzugefuegten Knoten gespeichert werden.      */
/*                                                                                     */
/* Rueckgabewert:                                                                      */
/*  - Fehlercode:   0 ... kein Fehler                                                  */
/*                  1 ... Es kann kein Knoten hinzugefuegt werden, weil kein Speicher  */
/*                        alloziiert werden kann!                                      */
/*                                                                                     */
/* Vorgehensweise:                                                                     */
/*  - Speicherplatz fuer den neuen Knoten dynamisch anfordern. Der Zeiger pNewNode     */
/*    zeigt dabei auf die Adresse im Arbeitsspeicher wo sich dieser neue Knoten be-    */
/*    findet. Ist der Inhalt des Zeigers pNewNode aber 0 (NULL) so konnte kein         */
/*    Speicherplatz angefordert werden.                                                */
/*  - War das Speicheranfordern nicht erfolgreich (pNewNode = 0), den entsprechenden   */
/*    Fehlercode dem aufrufenden Programm zurueckgeben.                                */
/*  - War das Speicheranfordern erfolgreich (pNewNode > 0):                            */
/*      - Die zu speichernden Daten im neuen Knoten (pNewNode) speichern (mit Hilfe    */
/*        der Memberfunktion SetValue der Klasse CNode)                                */
/*      - Das Verketten der Knoten ist nun abhaengig davon, an welcher Position sich   */
/*        der neue Knoten befindet:                                                    */
/*                                                                                     */
/*          - FALL 1: Der neue Knoten (pNewNode) befindet sich am Anfang der Liste,    */
/*            es ist daher auch das erste Element der Liste                            */
/*            (Kennzeichen: m_pRoot = 0 (NULL)                                         */
/*            Da es das erste Element der Liste ist gilt fuer diesen Knoten:           */
/*              - Der Zeiger auf den vorhergehende Knoten muss den Wert 0 (NULL)       */
/*                beinhalten (pNewNode->SetLast(0))                                    */
/*              - Der Zeiger auf das naechste Element muss ebenfalls den Wert 0 (NULL) */
/*                beinhalten (pNewNode->SetNext(0))                                    */
/*              - Die (Member-)Zeiger m_pRoot, m_pEnd und m_pActuell zeigen auf diesen */
/*                ersten Knoten.                                                       */
/*                                                                                     */
/*          - FALL 2: Der neue Knoten (pNewNode) befindet sich am Ende der Liste,      */
/*            (Kennzeichen: m_pActuell->GetNext() == 0 (NULL)                          */
/*            Da es der letzte Knoten der Liste ist gilt fuer diesen Knoten:           */
/*              - Der Zeiger von pNewNode auf den naechsten Knoten muss den Wert 0     */
/*                (NULL) beinhalten (pNewNode->SetNext(0))                             */
/*              - Der Zeiger von pNewNode auf den vorhergehenden Knoten muss den       */
/*                Zeiger pActuell beinhalten (pNewNode->SetLast(m_pActuell))           */
/*              - Der Zeiger von pActuell auf den naechste Knoten muss den Wert des    */
/*                Zeigers pNewNode beinhalten (m_pActuell->SetNext(pNewNode))          */
/*              - Die (Member-)Zeiger m_pEnd und m_pActuell zeigen auf diesen letzten  */
/*                Knoten.                                                              */
/*                                                                                     */
/*          - FALL 3: Der neue Knoten befindet sich nicht am Ende der Liste, sondern   */
/*            mittendrin                                                               */
/*            Da es ein Knoten innerhalb der Liste ist gilt fuer dieses Knoten:        */
/*              - Den auf pActuell folgenden Knoten in pTemp sichern                   */
/*              - Den neuen Knoten pNewNode nun so mit pActuell und pTemp verketten,   */
/*                dass es sich anschliessend zwischen pActuell und pTemp befindet      */
/*                  - Verkettung zwischen pActuell und pNewNode herstellen             */
/*                  - Verkettung zwischen pNewNode und pTemp herstellen                */
/*              - Der (Member-)Zeiger m_pActuell zeigt auf den eingefuegten Knoten     */
/*                                                                                     */
/* Veraenderte Member-Variablen:                                                       */
/*  - m_pRoot                                                                          */
/*  - m_pEnd                                                                           */
/*  - m_pActuell                                                                       */
/***************************************************************************************/
int CLinkedList::AddNode(int nValue)
{
    CNode *pNewNode;
    CNode *pTemp;       // nur fuer Fall 3 notwendig


    pNewNode = new CNode();

    if (pNewNode)
    {
        // Ist der Ruechgabewert pNewNode groesser als 0, so war die Anforderung 
        // des Speicherplatz erfolgreich. pNewNode beinhaltet nun die Startadresse 
        // des neuen Knotens.
        pNewNode->SetValue(nValue);

        if (m_pRoot == 0)
        {
            // Fall 1: Erster Knoten
            pNewNode->SetLast(0);
            pNewNode->SetNext(0);

            m_pRoot = pNewNode;
            m_pEnd = pNewNode;
        }
        else if (m_pActuell->GetNext() == 0)
        {
            // Fall 2: Knoten am Ende der Liste anhaengen

            // Die Knoten m_pActuell und pNewNode miteinander verbinden (Neuen Knoten
            // an das Ende der Liste, also an den "aktuellen" Knoten anhaengen
            pNewNode->SetNext(0);
            pNewNode->SetLast(m_pActuell);

            m_pActuell->SetNext(pNewNode); 

            m_pEnd = pNewNode;
        }
        else
        {
            // Fall 3: Knoten irgendwo zwischen Anfang und Ende

            // Adresse des auf pActuell folgenden Knoten in pTemp zwischenspeichern
            pTemp = m_pActuell->GetNext();
    
            // Den neuen Knoten zwischen pActuell und pTemp einfuegen (verketten)
            // pActuell <-> pNewNode
            m_pActuell->SetNext(pNewNode);
            pNewNode->SetNext(m_pActuell);

            // pNewNode <-> pTemp
            pNewNode->SetNext(pTemp);
            pTemp->SetLast(pNewNode);
        }

        // (Member)-Zeiger m_pActuell auf den neuen Knoten setzen
        m_pActuell = pNewNode;

        return (0); // Rueckgabewert 0 (alles okay, kein Fehler)

    }
    else
    {
        // Ist der Ruechgabewert pNewNode 0, so konnte kein Speicherplatz fuer 
        // den neuen Knoten angefordert werden.-> Fehlercode (1) zurueckgeben
        return (1);
    }
}


/***************************************************************************************/
/* int ShowList(void):                                                                 */
/*                                                                                     */
/* Aufgabe:                                                                            */
/*  - Die gesamte Liste ausgeben                                                       */
/*                                                                                     */
/* Uebergabeparameter:                                                                 */
/*  - keiner                                                                           */
/*                                                                                     */
/* Rueckgabewert:                                                                      */
/*  - Fehlercode:   0 ... kein Fehler                                                  */
/*                  1 ... Es ist keine Liste vorhanden                                 */
/*                                                                                     */
/* Vorgehensweise:                                                                     */
/*  - Ist die Liste leer (m_pRoot = 0) nur den entsprechenden Fehlercode zurueckgeben  */
/*  - Andernfalls mit Hilfe eines Hilfszeigers (hier pShowNode) und einer Schleife be- */
/*    ginnend beim ersten Knoten dessen Inhalt mit Hilfe von GetValue ausgeben.        */
/*                                                                                     */
/* Veraenderte Member-Variablen:                                                       */
/*  - keine                                                                            */
/***************************************************************************************/
int CLinkedList::ShowList(void)
{
    CNode *pShowNode;


    if (m_pRoot)
    {
        // Wenn eine Liste vorhanden ist (m_pRoot > 0)
        pShowNode = m_pRoot;
        do
        {
            cout << pShowNode->GetValue() << endl;

            pShowNode = pShowNode->GetNext();
        }
        while (pShowNode != 0);

        return (0); // Rueckgabewert: alles okay
    }
    else
    {
        // Wenn keine Liste vorhanden ist (m_pRoot = 0)
        return (1); // Rueckgabewert: Fehler
    }
}


/***************************************************************************************/
/* int RemoveNode(void):                                                               */
/*                                                                                     */
/* Aufgabe:                                                                            */
/*  Den Knoten auf welches der (Member-) Zeiger (m_pActuell) zeigt, aus der Liste ent- */
/*  fernen, und den Speicherplatz fuer diesen Knoten wieder freigeben.                 */
/*                                                                                     */
/* Uebergabeparameter:                                                                 */
/*  - keiner                                                                           */
/*                                                                                     */
/* Rueckgabewert:                                                                      */
/*  - Fehlercode:   0 ... kein Fehler                                                  */
/*                  1 ... Es ist kein Knoten vorhanden                                 */
/*                                                                                     */
/* Vorgehensweise:                                                                     */
/*  - Pruefen, ob ueberhaupt schon ein Eintrag existiert (m_pRoot > 0)                 */
/*  - Ist kein Knoten vorhanden (m_pRoot = 0), den entsprechenden Fehlercode dem auf-  */
/*    rufendem Programm zurueckgeben.                                                  */
/*  - Ist zumindest ein Knoten vorhanden (m_pRoot > 0) den Knoten, auf den der         */
/*    Zeiger pActuell zeigt loeschen. Beim Entfernen (loeschen) eines Knotens muss     */
/*    zwischen 4 Faellen unterschieden werden:                                         */
/*                                                                                     */
/*      - FALL 1: Die gesamte Liste besteht nur aus einem einzigen Knoten. Dieser      */
/*        Knoten soll nun geloescht werden.                                            */
/*        Kennzeichen: m_pActuell -> GetLast = 0                                       */
/*                     m_pActuell -> GetNext = 0                                       */
/*          - Den Speicherplatz mit der Adresse m_pActuell freigeben.                  */
/*          - Die (Member-)Zeiger m_pRoot, m_pEnd und m_pActuell mit 0 laden, da die   */
/*            Liste nun wieder leer ist.                                               */
/*                                                                                     */
/*      - FALL 2: Die Liste besteht aus mehreren Knoten, davon wird der erste Knoten   */
/*        geloescht.                                                                   */
/*        Kennzeichen: m_pActuell -> GetLast = 0                                       */
/*                     m_pActuell -> GetNext ungleich 0                                */
/*          - Den (Member-)Zeiger m_pActuell auf den auf m_pActuell folgenden Knoten   */
/*            setzen.                                                                  */
/*          - Den ersten Knoten (= m_pRoot) entfernen (loeschen) und den Speicherplatz */
/*            fuer diesen Knoten wieder freigeben.                                     */
/*          - Den Zeiger auf den vor dem aktuellen Knoten (m_pActuell) zeigenden mit 0 */
/*            (NULL) laden.                                                            */
/*          - Den (Member-)Zeiger m_pRoot mit m_pActuell laden, da die Liste nun einen */
/*            neuen Beginn (Wurzel) besitzt.                                           */
/*                                                                                     */
/*      - FALL 3: Die Liste besteht aus mehreren Knoten, davon wird der letzte Knoten  */
/*        geloescht.                                                                   */
/*        Kennzeichen: m_pActuell -> GetLast ungleich 0                                */
/*                     m_pActuell -> GetNext = 0                                       */
/*          - Den (Member-)Zeiger m_pActuell auf den von m_pActuell vorhergehenden     */
/*            Knoten setzen.                                                           */
/*          - Den letzten Knoten (= m_pEnd) entfernen (loeschen) und den Speicherplatz */
/*            fuer diesen Knoten wieder freigeben.                                     */
/*          - Den vom aktuellen Knoten (m_pActuell) aus auf den naechsten Knoten       */
/*            zeigenden Zeiger mit 0 (NULL) laden, da es keinen nachfolgenen Knoten    */
/*            gibt, bzw. der aktuelle Knoten auch gleichzeitig der letzte Knoten ist.  */
/*          - Den (Member-)Zeiger m_pEnd mit m_pActuell laden, da die Liste nun ein    */
/*            neues Ende besitzt.                                                      */
/*                                                                                     */
/*      - FALL 4: Die Liste besteht aus mehreren Knoten, davon wird weder der erste    */
/*        Knoten noch der letzte Knoten geloescht. Sondern ein Knoten irgendwo         */
/*        zwischen Ersten und Letztem.                                                 */
/*        Kennzeichen: m_pActuell -> GetLastEntry ungleich 0                           */
/*                     m_pActuell -> GetNextEntry ungleich 0                           */
/*          - Die Adresse des auf m_pActuell folgenden Knotens in pTempNach sichern,   */
/*            und den zu m_pActuell vorherigen Eintrag in pTempVor sichern.            */
/*          - Den Knoten auf den der Zeiger m_pActuell zeigt entfernen (loeschen) und  */
/*            den Speicherplatz fuer diesen Knoten wieder freigeben.                   */
/*          - Die Knoten pTempVor und pTempNach miteinander verketten.                 */
/*          - Den (Member-)Zeiger m_pActuell mit pTempVor laden                        */
/*                                                                                     */
/* Veraenderte Member-Variablen:                                                       */
/*  - m_pRoot                                                                          */
/*  - m_pEnd                                                                           */
/*  - m_pActuell                                                                       */
/***************************************************************************************/
int CLinkedList::RemoveNode(void)
{
    CNode *pTempVor;    // Fuer Fall 4 (das zu loeschende Entry befindet sich
    CNode *pTempNach;   //   in der Mitte der Liste)


    if (m_pRoot == NULL)
    {
        // Wenn kein Knoten vorhanden ist (m_pRoot = 0)
        return (1); // Rueckgabewert: Fehler
    }
    else
    {
        if (m_pActuell->GetLast() == NULL)
        {
            if (m_pActuell->GetNext() == NULL)
            {
                // Fall 1: Die Liste besteht nur aus einem einzigen Knoten
                // Dieser Knoten wird nun geloescht, der Speicherplatz fuer
                // diesen Knoten wird wieder freigegeben und die Zeiger m_pRoot,
                // m_pActuell und m_pEnd werden geloescht
                delete (m_pActuell);

                m_pRoot = 0;
                m_pActuell = 0;
                m_pEnd = 0;
            }
            else
            {
                // Fall 2: Die Liste besteht aus mehreren Knoten.
                // Der erste Knoten wird nun geloescht, und der Speicherplatz fuer
                // dieses Knoten wird wieder freigegeben

                // Zeiger pActuell auf den naechsten Knoten setzen
                m_pActuell = m_pActuell->GetNext();

                // Den ersten Knoten (m_pRoot) loeschen und den Speicherplatz fuer
                //  diesen Knoten wieder freigeben
                delete (m_pRoot);

                // Den Zeiger m_pActuell->SetLast() mit NULL laden, da ja nun 
                // dieser Knoten das erste Entry der Liste ist
                m_pActuell->SetLast(0);

                // Den Zeiger m_pRoot mit m_pActuell laden, da die Liste nun einen
                // neuen Beginn (Wurzel) besitzt
                m_pRoot = m_pActuell;
            }
        }
        else
        {
            if (m_pActuell->GetNext() == NULL)
            {
                // Fall 3: Die Liste besteht aus mehreren Knoten.
                // Der letzte Knoten wird nun geloescht, und der Speicherplatz
                // fuer diesen Knoten wird wieder freigegeben

                // Zeiger m_pActuell auf den vorherige Knoten setzen
                m_pActuell = m_pActuell->GetLast();

                // Den letzten Knoten (m_pEnd) loeschen,
                // den Speicherplatz fuer dieses Knoten wieder freigeben
                delete (m_pEnd);

                // Den Zeiger m_pActuell->SetNext mit NULL laden, da ja nun 
                // dieser Knoten der letzte Knoten der Liste ist.
                m_pActuell->SetNext(0);

                // Den Zeiger m_pEnd mit m_pActuell laden, da die Liste nun
                // ein neues Ende besitzt
                m_pEnd = m_pActuell;
            }
            else
            {
                // Fall 4: Die Liste besteht aus mehreren Knoten.
                // Es wird weder der erste noch der letzte Knoten geloescht,
                // sondern ein Knoten irgendwo in der Mitte der Liste, der
                // Speicherplatz fuer diesen Knoten wird wieder freigegeben.

                // Die Adresse des vorhergehenden Knotens in pTempVor sichern,
                // und die Adresse des naechsten Knotens in pTempNach sichern.
                // (Anm.: Bezueglich m_pActuell)
                pTempVor = m_pActuell->GetLast();
                pTempNach = m_pActuell->GetNext();

                // Den Knoten m_pAktuell loeschen, der Speicherplatz fuer
                // diesen Knoten wird wieder freigegeben
                delete (m_pActuell);

                // Die Knoten pTempVor und pTempNach verketten
                pTempVor->SetNext(pTempNach);
                pTempNach->SetLast(pTempVor);

                // Den Zeiger m_pActuell mit pTempVor laden
                m_pActuell = pTempVor;
            }
        }
    }

    return (0); // Rueckgabewert: alles okay
}


/***************************************************************************************/
/* int DeleteList(void):                                                               */
/*                                                                                     */
/* Aufgabe:                                                                            */
/*  - Die gesamte Liste vom Speicher entfernen                                         */
/*                                                                                     */
/* Uebergabeparameter:                                                                 */
/*  - keiner                                                                           */
/*                                                                                     */
/* Rueckgabewert:                                                                      */
/*  - Fehlercode:   0 ... kein Fehler                                                  */
/*                  1 ... Es ist keine Liste vorhanden                                 */
/*                                                                                     */
/* Vorgehensweise:                                                                     */
/*  - Ist die Liste leer (m_pRoot = 0) muss sie auch nicht geloescht werden!! - Was    */
/*    nicht vorhanden ist, kann auch nicht geloescht werden!!                          */
/*  - Andernfalls mit Hilfe zweier Hilfszeigers (hier pDeleteNode und pDeleteNext)     */
/*    beginnend beim ersten Knoten diesen mit Hilfe einer Schleife Knoten fuer Knoten  */
/*    vom Speicher entfernen. Dabei zeigt der Zeiger pDeleteNode immer auf den zu ent- */
/*    fernenden Knoten und der Zeiger pDeleteNext auf den darauf folgenden Knoten.     */
/*    Dieser Vorgang wird solange fortgesetzt, bis pDeleteNext ein "Null-Zeiger" ist,  */
/*    D.h. der Zeiger den Wert Null beinhaltet. Denn dann ist er am Ende der Liste an- */
/*    gekommen.                                                                        */
/*                                                                                     */
/* Veraenderte Member-Variablen:                                                       */
/*  - m_pRoot                                                                          */
/*  - m_pEnd                                                                           */
/*  - m_pActuell                                                                       */
/***************************************************************************************/
int CLinkedList::DeleteList(void)
{
    CNode *pDeleteNode;
    CNode *pDeleteNext;


    // Liste nur dann loeschen, wenn eine vorhanden ist!
    if (m_pRoot != 0)
    {
        // Zeiger pDeleteNode auf den ersten Knoten setzen
        pDeleteNode = m_pRoot;

        do
        {
            // Zeiger pDeleteNext auf den naechsten Knoten setzen
            pDeleteNext = pDeleteNode->GetNext();

            // Knoten auf den der Zeiger pDeleteNode zeigt loeschen
            delete (pDeleteNode);

            // Zeiger pDeleteNode auf den naechsten Knoten setzen
            pDeleteNode = pDeleteNext;
        } while (pDeleteNode);

        // Die Membervariablen ebenfalls loeschen
        m_pRoot = 0;
        m_pEnd = 0;
        m_pActuell = 0;

        return (0); // Rueckgabewert: alles okay
    }
    else
    {
        // Wenn keine Liste vorhanden ist (m_pRoot = 0)
        return (1); // Rueckgabewert: Fehler
    }
}


/***************************************************************************************/
/* int Scroll (int nDirection):                                                        */
/*                                                                                     */
/* Aufgabe:                                                                            */
/*  - In der Liste entweder um einen Knoten vor oder zurueck scrollen, oder an den     */
/*    Beginn oder das Ende der Liste scrollen. Der Uebergabeparameter gibt dabei die   */
/*    Scrollrichtung an..                                                              */
/*                                                                                     */
/* Uebergabeparameter:                                                                 */
/*  - nDirection: gibt die Scrollrichtung an                                           */
/*                  - zum Anfang der Liste                                             */
/*                  - zum Ende der Liste                                               */
/*                  - zum vorhergehenden Knoten                                        */
/*                  - zum naechsten Knoten                                             */
/*                                                                                     */
/* Rueckgabewert:                                                                      */
/*  - Fehlercode:   0 ... kein Fehler                                                  */
/*                  1 ... Es ist keine Liste vorhanden                                 */
/*                  2 ... Scrollbewegung nach vor nicht moeglich                       */
/*                  3 ... Scrollbewegung zurueck nicht moeglich                        */
/*                                                                                     */
/* Vorgehensweise:                                                                     */
/*  - Fuer diese Funktionalitaet eignet sich sehr gut ein "switch"-Anweisung, wobei    */
/*    die einzelnen Faelle mit je einer Konstante - in Form einer enum-Konstante       */
/*    siehe Beginn des Codes) - realisiert wurde.                                      */
/*  - Zu Beginn muss aber ueberprueft werden, ob ueberhaupt eine Liste vorhanden ist.  */
/*  - Befindet sich der zu scrollende Zeiger m_pActuell am Ende der Liste, so kann sie */
/*    nicht mehr weiter nach vor gescrollt werden. Fuer diesen Fall muss ein Fehler-   */
/*    code an das aufrufende Programm zurueckgegeben werden. Analoges gilt auch, wenn  */
/*    der Zeiger m_pActuell am Beginn der Liste befindet.                              */
/*                                                                                     */
/* Veraenderte Member-Variablen:                                                       */
/*  - m_pActuell                                                                       */
/***************************************************************************************/
int CLinkedList::Scroll(int nDirection)
{
    int ErrorCode;


    if (m_pRoot)
    {
        // Es ist eine Liste vorhanden
        switch(nDirection)
        {
        case SCROLL_BACK:
            if (m_pActuell->GetLast())
            {
                m_pActuell = m_pActuell->GetLast();
                ErrorCode = 0;
            }
            else
            {
                // Fehler:
                ErrorCode = 2;
            }
            break;

        case SCROLL_FORWARD:
            if (m_pActuell->GetNext())
            {
                m_pActuell = m_pActuell->GetNext();
                ErrorCode = 0;
            }
            else
            {
                // Fehler: 
                ErrorCode = 1;
            }
            break;

        case SCROLL_BEGIN:
            m_pActuell = m_pRoot;
            ErrorCode = 0;
            break;

        case SCROLL_END:
            m_pActuell = m_pEnd;
            ErrorCode = 0;
            break;
        }
    }
    else
    {
        // es ist keine Liste vorhande, Fehlercode (1) zurueckgeben
        ErrorCode = 1;
    }

    return (ErrorCode);
}

